/*-----------------------------------------------------------------------------------------------*/
/*  Acquisition et transmission en RS232 des données de deux codeurs absolus communiquant en SSI */                 
/*                            Ismaïl EL HASNAOUI & Benoit CONFLAND                               */
/*                                PFE IMA 5 Polytech'Lille                                       */
/*-----------------------------------------------------------------------------------------------*/

int binaryArray_av[14];                     
int binaryArray_ar[14];                     
int i = 0;            
int j = 0;           
int pos_av=0;       
int pos_ar=0;         


void getPosition();              //  Récupération des données, communication SSI avec les capteurs
void transformation();           //  Mise en forme de la donnée, on enléve le bit de start et on passe du binaire en décimal
void sendPosition();             //  Envoi des données provenant des deux capteurs  en RS232,
                                 //  les signaux émis par l'arduino sont en TTL une conversion TTL -> RS232 est nécessaire à l'aide d'un MAX232 par exemple.

void setup()
{
  
  SetParity('n');                // Initialisation de la liaison RS232
  SetStopBits(1);
  SetWordLength(8);
  
 
  
  DDRB= B00000000;               //Initialisation du PortB, Utilisation des pins 12 (PB6) et 10 (PB4) en sortie (Clk) 
  DDRB = (1<<PB6)|(1<<PB4);      //Pins 11 et 13 (Data)  
  
  delay(500);
  Serial1.begin(9600);  
}


void loop()
{
  getPosition();                                
  transformation();                              
  sendPosition();                                
  delay(2);
}


void getPosition()
{
  for(j = 0;j < 14; j++)          
  {
     binaryArray_av[j] = 0; 
     binaryArray_ar[j] = 0; 
  }
  
  delayMicroseconds(40);
  
  for(j = 0; j < 14; j++)                         // Les codeurs absolus transmettent des données en 13 bits
  {    
    PORTB = (0<<PB4)|(0<<PB6);                    // Géneration de l'horloge CLK à 0
    delayMicroseconds(3);                         // Le premier bit reçu correspond à un bit de start qui ne fait pas partie de la donnée
    
    binaryArray_ar[j] = (PINB & (1<<7));          // A chaque coup d'horloge les capteurs transmettent un bit, 
    binaryArray_av[j] = (PINB & (1<<5));          // Lecture les deux pins et on stock la donnée de chaque capteur 
                                                  // dans son tableau. 
   
  
    PORTB = (1<<PB4)|(1<<PB6);                    // Géneration de l'horloge CLK à 1
    delayMicroseconds(3);
  }
  
  delayMicroseconds(100); 
}


void transformation() 
{
  for (i=0; i < 14; i++)                          // Remise en forme de l'information, pour les capteurs une valeur différente de 0 correspond à 1. 
  {
    if (binaryArray_av[i]!=0) 
      binaryArray_av[i]=1;
    
    if (binaryArray_ar[i]!=0) 
       binaryArray_ar[i]=1;  
  }
  
  pos_av = 0;
  pos_ar = 0;
  
 for (i = 1; i < 14; i ++)                        // Suppression du bit de start
 {
   pos_av = (pos_av<<1) | (binaryArray_av[i]);
   pos_ar = (pos_ar<<1) | (binaryArray_ar[i]);
 }
   
}


void sendPosition()
{
  
  
  /*Les données du capteurs   */
  int byte1 = pos_av/256;                        // La liaison RS232 nécessite des trames de 8 bits 
  int byte2 = pos_av - byte1*256;                // Découpage de la donnée en deux octets 
  
  int byte3 = pos_ar/256;                        
  int byte4 = pos_ar - byte3*256;
 

  Serial1.write(byte2);             
  delay(1);  
  Serial1.write(byte1);  
  
  delay (20);
  
  Serial1.write(byte4);  
  delay(1);  
  Serial1.write(byte3+32);                      // Signature de l'octet de poid fort afin d'identifier les données provenant du capteur arrière 
  
}


void SetStopBits(int nbStopBits)
{
  if (nbStopBits == 1)
    UCSR0C = UCSR0C | B00000100;
  else 
    UCSR0C = UCSR0C | B00000000;
}


void SetParity(int parity)
{
  if ((parity == 'O')|(parity == 'o'))
    UCSR0C = UCSR0C | B00110000;
  
  else if ((parity == 'E')|(parity == 'e'))
    UCSR0C = UCSR0C | B00100000;
  
  else // ((parity == 'N')|(parity == 'n')))
    UCSR0C = UCSR0C | B00000000;
}


void SetWordLength(int wordlength)
{
  
  if (wordlength == 5)  
    UCSR0C = UCSR0C | B00000000;
 
  else if (wordlength == 6)
    UCSR0C = UCSR0C | B00000010;
   
  else if (wordlength == 7)
    UCSR0C = UCSR0C | B00000100;
  
  else if (wordlength == 9){
    UCSR0C = UCSR0C | B00000110;
    UCSR0B = UCSR0B | B00000100;
  }
  else  
    UCSR0C = UCSR0C | B00000110;
 
}


